@@ -19,7 +19,12 @@ import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
 import net.bytebuddy.agent.utility.nullability.AlwaysNull;
 import net.bytebuddy.agent.utility.nullability.MaybeNull;
 
-import java.io.*;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileOutputStream;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.OutputStream;
 import java.lang.instrument.Instrumentation;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
@@ -30,8 +35,16 @@ import java.net.URLClassLoader;
 import java.security.CodeSource;
 import java.security.PrivilegedAction;
 import java.security.ProtectionDomain;
-import java.util.*;
-import java.util.jar.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collections;
+import java.util.List;
+import java.util.Locale;
+import java.util.jar.Attributes;
+import java.util.jar.JarEntry;
+import java.util.jar.JarInputStream;
+import java.util.jar.JarOutputStream;
+import java.util.jar.Manifest;
 
 /**
  * <p>
@@ -137,11 +150,6 @@ public class ByteBuddyAgent {
      */
     private static final String OS_NAME = "os.name";
 
-    /**
-     * Represents the {@code file} URL protocol.
-     */
-    private static final String FILE_PROTOCOL = "file";
-
     /**
      * The attachment type evaluator to be used for determining if an attachment requires an external process.
      */
@@ -623,6 +631,7 @@ public class ByteBuddyAgent {
      * @param argument           The argument to provide to the agent or {@code null} if no argument should be supplied.
      * @throws Exception If an exception occurs during the attachment or the external process fails the attachment.
      */
+    @SuppressFBWarnings(value = "OS_OPEN_STREAM_EXCEPTION_PATH", justification = "Outer stream holds file handle and is closed")
     private static void installExternal(AttachmentProvider.Accessor.ExternalAttachment externalAttachment,
                                         String processId,
                                         File agent,
@@ -637,8 +646,9 @@ public class ByteBuddyAgent {
                 }
                 try {
                     attachmentJar = File.createTempFile(ATTACHER_FILE_NAME, JAR_FILE_EXTENSION);
-                    JarOutputStream jarOutputStream = new JarOutputStream(new FileOutputStream(attachmentJar));
+                    OutputStream outputStream = new FileOutputStream(attachmentJar);
                     try {
+                        JarOutputStream jarOutputStream = new JarOutputStream(outputStream);
                         jarOutputStream.putNextEntry(new JarEntry(Attacher.class.getName().replace('.', '/') + CLASS_FILE_EXTENSION));
                         byte[] buffer = new byte[1024 * 8];
                         int index;
@@ -646,8 +656,9 @@ public class ByteBuddyAgent {
                             jarOutputStream.write(buffer, 0, index);
                         }
                         jarOutputStream.closeEntry();
-                    } finally {
                         jarOutputStream.close();
+                    } finally {
+                        outputStream.close();
                     }
                 } finally {
                     inputStream.close();
@@ -705,7 +716,7 @@ public class ByteBuddyAgent {
                 return null;
             }
             URL location = codeSource.getLocation();
-            if (!location.getProtocol().equals(FILE_PROTOCOL)) {
+            if (!location.getProtocol().equals("file")) {
                 return null;
             }
             try {
@@ -1451,7 +1462,7 @@ public class ByteBuddyAgent {
                     return null;
                 }
                 URL location = codeSource.getLocation();
-                if (!location.getProtocol().equals(FILE_PROTOCOL)) {
+                if (!location.getProtocol().equals("file")) {
                     return null;
                 }
                 File agentJar;
@@ -1464,26 +1475,29 @@ public class ByteBuddyAgent {
                     return null;
                 }
                 // It is necessary to check the manifest of the containing file as this code can be shaded into another artifact.
-                JarInputStream jarInputStream = new JarInputStream(new FileInputStream(agentJar));
+                Manifest manifest;
+                InputStream inputStream = new FileInputStream(agentJar);
                 try {
-                    Manifest manifest = jarInputStream.getManifest();
-                    if (manifest == null) {
-                        return null;
-                    }
-                    Attributes attributes = manifest.getMainAttributes();
-                    if (attributes == null) {
-                        return null;
-                    }
-                    if (installer.getName().equals(attributes.getValue(AGENT_CLASS_PROPERTY))
-                            && Boolean.parseBoolean(attributes.getValue(CAN_REDEFINE_CLASSES_PROPERTY))
-                            && Boolean.parseBoolean(attributes.getValue(CAN_RETRANSFORM_CLASSES_PROPERTY))
-                            && Boolean.parseBoolean(attributes.getValue(CAN_SET_NATIVE_METHOD_PREFIX))) {
-                        return agentJar;
-                    } else {
-                        return null;
-                    }
-                } finally {
+                    JarInputStream jarInputStream = new JarInputStream(inputStream);
+                    manifest = jarInputStream.getManifest();
                     jarInputStream.close();
+                } finally {
+                    inputStream.close();
+                }
+                if (manifest == null) {
+                    return null;
+                }
+                Attributes attributes = manifest.getMainAttributes();
+                if (attributes == null) {
+                    return null;
+                }
+                if (installer.getName().equals(attributes.getValue(AGENT_CLASS_PROPERTY))
+                        && Boolean.parseBoolean(attributes.getValue(CAN_REDEFINE_CLASSES_PROPERTY))
+                        && Boolean.parseBoolean(attributes.getValue(CAN_RETRANSFORM_CLASSES_PROPERTY))
+                        && Boolean.parseBoolean(attributes.getValue(CAN_SET_NATIVE_METHOD_PREFIX))) {
+                    return agentJar;
+                } else {
+                    return null;
                 }
             }
 
@@ -1493,6 +1507,7 @@ public class ByteBuddyAgent {
              * @return The agent jar file.
              * @throws IOException If an I/O exception occurs.
              */
+            @SuppressFBWarnings(value = "OS_OPEN_STREAM_EXCEPTION_PATH", justification = "Outer stream holds file handle and is closed")
             private static File createJarFile() throws IOException {
                 InputStream inputStream = Installer.class.getResourceAsStream('/' + Installer.class.getName().replace('.', '/') + CLASS_FILE_EXTENSION);
                 if (inputStream == null) {
@@ -1507,8 +1522,9 @@ public class ByteBuddyAgent {
                     manifest.getMainAttributes().put(new Attributes.Name(CAN_REDEFINE_CLASSES_PROPERTY), Boolean.TRUE.toString());
                     manifest.getMainAttributes().put(new Attributes.Name(CAN_RETRANSFORM_CLASSES_PROPERTY), Boolean.TRUE.toString());
                     manifest.getMainAttributes().put(new Attributes.Name(CAN_SET_NATIVE_METHOD_PREFIX), Boolean.TRUE.toString());
-                    JarOutputStream jarOutputStream = new JarOutputStream(new FileOutputStream(agentJar), manifest);
+                    OutputStream outputStream = new FileOutputStream(agentJar);
                     try {
+                        JarOutputStream jarOutputStream = new JarOutputStream(outputStream, manifest);
                         jarOutputStream.putNextEntry(new JarEntry(Installer.class.getName().replace('.', '/') + CLASS_FILE_EXTENSION));
                         byte[] buffer = new byte[1024 * 8];
                         int index;
@@ -1516,8 +1532,9 @@ public class ByteBuddyAgent {
                             jarOutputStream.write(buffer, 0, index);
                         }
                         jarOutputStream.closeEntry();
-                    } finally {
                         jarOutputStream.close();
+                    } finally {
+                        outputStream.close();
                     }
                     return agentJar;
                 } finally {
