@@ -17,6 +17,7 @@ package net.bytebuddy.utility.dispatcher;
 
 import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
 import net.bytebuddy.ClassFileVersion;
+import net.bytebuddy.asm.AsmVisitorWrapper;
 import net.bytebuddy.build.AccessControllerPlugin;
 import net.bytebuddy.build.HashCodeAndEqualsPlugin;
 import net.bytebuddy.description.method.MethodDescription;
@@ -34,8 +35,19 @@ import org.objectweb.asm.Type;
 import java.io.File;
 import java.io.FileOutputStream;
 import java.io.OutputStream;
-import java.lang.annotation.*;
-import java.lang.reflect.*;
+import java.lang.annotation.Annotation;
+import java.lang.annotation.Documented;
+import java.lang.annotation.ElementType;
+import java.lang.annotation.Retention;
+import java.lang.annotation.RetentionPolicy;
+import java.lang.annotation.Target;
+import java.lang.reflect.Array;
+import java.lang.reflect.Constructor;
+import java.lang.reflect.InvocationHandler;
+import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
+import java.lang.reflect.Modifier;
+import java.lang.reflect.Proxy;
 import java.security.Permission;
 import java.security.PrivilegedAction;
 import java.util.HashMap;
@@ -147,7 +159,7 @@ public class JavaDispatcher<T> implements PrivilegedAction<T> {
      * @param <T>         The resolved type.
      * @return An action for creating an appropriate dispatcher.
      */
-    protected static <T> PrivilegedAction<T> of(Class<T> type, @MaybeNull ClassLoader classLoader) {
+    public static <T> PrivilegedAction<T> of(Class<T> type, @MaybeNull ClassLoader classLoader) {
         return of(type, classLoader, GENERATE);
     }
 
@@ -180,9 +192,12 @@ public class JavaDispatcher<T> implements PrivilegedAction<T> {
         try {
             Object securityManager = System.class.getMethod("getSecurityManager").invoke(null);
             if (securityManager != null) {
+                Object permission = Class.forName("java.lang.RuntimePermission")
+                        .getConstructor(String.class)
+                        .newInstance("net.bytebuddy.createJavaDispatcher");
                 Class.forName("java.lang.SecurityManager")
                         .getMethod("checkPermission", Permission.class)
-                        .invoke(securityManager, new RuntimePermission("net.bytebuddy.createJavaDispatcher"));
+                        .invoke(securityManager, permission);
             }
         } catch (NoSuchMethodException ignored) {
             /* security manager not available on current VM */
@@ -197,6 +212,8 @@ public class JavaDispatcher<T> implements PrivilegedAction<T> {
             }
         } catch (IllegalAccessException exception) {
             throw new IllegalStateException("Failed to access security manager", exception);
+        } catch (InstantiationException exception) {
+            throw new IllegalStateException("Failed to instantiate runtime permission", exception);
         }
         Map<Method, Dispatcher> dispatchers = generate
                 ? new LinkedHashMap<Method, Dispatcher>()
@@ -1234,7 +1251,7 @@ public class JavaDispatcher<T> implements PrivilegedAction<T> {
          */
         @SuppressFBWarnings(value = {"REC_CATCH_EXCEPTION", "DP_CREATE_CLASSLOADER_INSIDE_DO_PRIVILEGED"}, justification = "Expected internal invocation.")
         protected static Object proxy(Class<?> proxy, Map<Method, Dispatcher> dispatchers) {
-            ClassWriter classWriter = new ClassWriter(0);
+            ClassWriter classWriter = new ClassWriter(AsmVisitorWrapper.NO_FLAGS);
             classWriter.visit(ClassFileVersion.JAVA_V5.getMinorMajorVersion(),
                     Opcodes.ACC_PUBLIC,
                     Type.getInternalName(proxy) + "$Proxy",
@@ -1310,8 +1327,8 @@ public class JavaDispatcher<T> implements PrivilegedAction<T> {
          */
         @SuppressFBWarnings(value = {"REC_CATCH_EXCEPTION", "DP_CREATE_CLASSLOADER_INSIDE_DO_PRIVILEGED"}, justification = "Expected internal invocation.")
         protected static Invoker invoker() {
-            ClassWriter classWriter = new ClassWriter(0);
-            classWriter.visit(ClassFileVersion.JAVA_V5.getMinorMajorVersion(),
+            ClassWriter classWriter = new ClassWriter(AsmVisitorWrapper.NO_FLAGS);
+            classWriter.visit(ClassFileVersion.ofThisVm().getMinorMajorVersion(),
                     Opcodes.ACC_PUBLIC,
                     Type.getInternalName(Invoker.class) + "$Dispatcher",
                     null,
