@@ -3,28 +3,43 @@ package net.bytebuddy.asm;
 import net.bytebuddy.ByteBuddy;
 import net.bytebuddy.description.method.MethodDescription;
 import net.bytebuddy.description.modifier.Visibility;
+import net.bytebuddy.description.type.TypeDescription;
 import net.bytebuddy.dynamic.ClassFileLocator;
 import net.bytebuddy.dynamic.DynamicType;
 import net.bytebuddy.dynamic.loading.ByteArrayClassLoader;
 import net.bytebuddy.dynamic.loading.ClassLoadingStrategy;
 import net.bytebuddy.implementation.FieldAccessor;
 import net.bytebuddy.implementation.bytecode.constant.NullConstant;
+import net.bytebuddy.matcher.ElementMatchers;
 import net.bytebuddy.pool.TypePool;
 import net.bytebuddy.test.packaging.MemberSubstitutionTestHelper;
+import net.bytebuddy.test.utility.JavaVersionRule;
+import net.bytebuddy.utility.JavaConstant;
+import org.junit.Rule;
 import org.junit.Test;
+import org.junit.rules.MethodRule;
 
+import java.util.Collections;
+import java.util.List;
 import java.util.concurrent.Callable;
 
 import static net.bytebuddy.matcher.ElementMatchers.any;
-import static net.bytebuddy.matcher.ElementMatchers.*;
+import static net.bytebuddy.matcher.ElementMatchers.isDeclaredBy;
+import static net.bytebuddy.matcher.ElementMatchers.named;
+import static net.bytebuddy.matcher.ElementMatchers.none;
+import static org.hamcrest.CoreMatchers.instanceOf;
 import static org.hamcrest.CoreMatchers.is;
-import static org.hamcrest.CoreMatchers.*;
+import static org.hamcrest.CoreMatchers.notNullValue;
+import static org.hamcrest.CoreMatchers.nullValue;
 import static org.hamcrest.MatcherAssert.assertThat;
 
 public class MemberSubstitutionTest {
 
     private static final String FOO = "foo", BAR = "bar", QUX = "qux", BAZ = "baz", RUN = "run";
 
+    @Rule
+    public MethodRule javaVersionRule = new JavaVersionRule();
+
     @Test
     public void testFieldReadStub() throws Exception {
         Class<?> type = new ByteBuddy()
@@ -672,6 +687,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainEmpty() throws Exception {
         Class<?> type = new ByteBuddy()
                 .redefine(FieldAccessSample.class)
@@ -688,6 +704,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainSimple() throws Exception {
         Class<?> type = new ByteBuddy()
                 .redefine(FieldAccessSample.class)
@@ -705,6 +722,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainFieldReadOriginal() throws Exception {
         Class<?> type = new ByteBuddy()
                 .redefine(FieldAccessSample.class)
@@ -722,6 +740,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainFieldWriteOriginal() throws Exception {
         Class<?> type = new ByteBuddy()
                 .redefine(FieldAccessSample.class)
@@ -739,6 +758,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainFieldReadStaticOriginal() throws Exception {
         Class<?> type = new ByteBuddy()
                 .redefine(StaticFieldAccessSample.class)
@@ -756,6 +776,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainFieldWriteStaticOriginal() throws Exception {
         Class<?> type = new ByteBuddy()
                 .redefine(StaticFieldAccessSample.class)
@@ -773,6 +794,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainVirtualMethodInvocationOriginal() throws Exception {
         Class<?> type = new ByteBuddy()
                 .redefine(MethodInvokeSample.class)
@@ -790,6 +812,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainStaticMethodInvocationOriginal() throws Exception {
         Class<?> type = new ByteBuddy()
                 .redefine(StaticMethodInvokeSample.class)
@@ -807,6 +830,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainSpecialMethodInvocationOriginal() throws Exception {
         Class<?> type = new ByteBuddy()
                 .redefine(VirtualMethodCallSubstitutionSample.Extension.class)
@@ -821,6 +845,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainConstructionOriginal() throws Exception {
         Class<?> type = new ByteBuddy()
                 .redefine(ConstructorSubstitutionSample.class)
@@ -834,6 +859,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainArgumentLoaded() throws Exception {
         Class<?> type = new ByteBuddy()
                 .redefine(ArgumentSample.class)
@@ -849,6 +875,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainArgumentInstrumentedMethodLoaded() throws Exception {
         Class<?> type = new ByteBuddy()
                 .redefine(ArgumentSample.class)
@@ -864,6 +891,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainArgumentInstrumentedMethodLoadedThis() throws Exception {
         Class<?> type = new ByteBuddy()
                 .redefine(ArgumentSample.class)
@@ -879,6 +907,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainFieldRead() throws Exception {
         Class<?> type = new ByteBuddy()
                 .redefine(FieldAccessSample.class)
@@ -897,6 +926,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainStaticFieldRead() throws Exception {
         Class<?> type = new ByteBuddy()
                 .redefine(StaticFieldAccessSample.class)
@@ -914,6 +944,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainFieldWrite() throws Exception {
         Class<?> type = new ByteBuddy()
                 .redefine(FieldAccessSample.class)
@@ -932,6 +963,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainFieldStaticWrite() throws Exception {
         Class<?> type = new ByteBuddy()
                 .redefine(StaticFieldAccessSample.class)
@@ -950,6 +982,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainMethodInvocation() throws Exception {
         Class<?> type = new ByteBuddy()
                 .redefine(FieldAccessSample.class)
@@ -968,6 +1001,26 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
+    public void testSubstitutionChainMethodInvocationWithArgument() throws Exception {
+        Class<?> type = new ByteBuddy()
+                .redefine(StaticMethodInvokeSample.class)
+                .visit(MemberSubstitution.strict().method(named(BAR)).replaceWithChain(
+                        new MemberSubstitution.Substitution.Chain.Step.ForInvocation.Factory(
+                                StaticMethodInvokeSample.class.getDeclaredMethod(BAZ, String.class))).on(named(RUN)))
+                .make()
+                .load(ClassLoadingStrategy.BOOTSTRAP_LOADER, ClassLoadingStrategy.Default.WRAPPER)
+                .getLoaded();
+        Object instance = type.getDeclaredConstructor().newInstance();
+        assertThat(type.getDeclaredField(BAR).get(instance), is((Object) BAR));
+        assertThat(type.getDeclaredField(BAZ).get(instance), is((Object) BAZ));
+        assertThat(type.getDeclaredMethod(RUN).invoke(instance), nullValue(Object.class));
+        assertThat(type.getDeclaredField(BAR).get(instance), is((Object) BAR));
+        assertThat(type.getDeclaredField(BAZ).get(instance), is((Object) FOO));
+    }
+
+    @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainStaticMethodInvocation() throws Exception {
         Class<?> type = new ByteBuddy()
                 .redefine(StaticFieldAccessSample.class)
@@ -985,6 +1038,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainTypeAssignment() throws Exception{
         Class<?> type = new ByteBuddy()
                 .redefine(StaticFieldAccessSample.class)
@@ -1003,6 +1057,7 @@ public class MemberSubstitutionTest {
     }
 
     @Test
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
     public void testSubstitutionChainTypeAssignmentExplicit() throws Exception{
         Class<?> type = new ByteBuddy()
                 .redefine(StaticFieldAccessSample.class)
@@ -1020,6 +1075,42 @@ public class MemberSubstitutionTest {
         assertThat(type.getDeclaredField(BAR).get(instance), is((Object) BAZ));
     }
 
+    @Test
+    @JavaVersionRule.Enforce(8)
+    public void testLambdaSubstitutionWithStub() throws Exception {
+        Class<?> type = new ByteBuddy()
+                .redefine(Class.forName("net.bytebuddy.test.precompiled.v8.LambdaSampleFactory"))
+                .visit(MemberSubstitution.strict()
+                        .lambdaExpression()
+                        .withName(ElementMatchers.is("call"))
+                        .withType(ElementMatchers.is(JavaConstant.MethodType.of(
+                                TypeDescription.ForLoadedType.of(Callable.class),
+                                Collections.<TypeDescription>emptyList())))
+                        .withArguments(ElementMatchers.<List<? extends JavaConstant>>any())
+                        .stub()
+                        .on(named("nonCapturing")))
+                .make()
+                .load(ClassLoadingStrategy.BOOTSTRAP_LOADER, ClassLoadingStrategy.Default.WRAPPER)
+                .getLoaded();
+        Object instance = type.getDeclaredConstructor().newInstance();
+        assertThat(type.getMethod("nonCapturing").invoke(instance), nullValue(Object.class));
+    }
+
+    @Test
+    @JavaVersionRule.Enforce(8)
+    @SuppressWarnings("unchecked") // In absence of @SafeVarargs
+    public void testLambdaSubstitutionWithChainOriginal() throws Exception {
+        Class<?> type = new ByteBuddy()
+                .redefine(Class.forName("net.bytebuddy.test.precompiled.v8.LambdaSampleFactory"))
+                .visit(MemberSubstitution.strict().lambdaExpression().replaceWithChain(
+                        MemberSubstitution.Substitution.Chain.Step.OfOriginalExpression.INSTANCE).on(named("nonCapturing")))
+                .make()
+                .load(ClassLoadingStrategy.BOOTSTRAP_LOADER, ClassLoadingStrategy.Default.WRAPPER)
+                .getLoaded();
+        Object instance = type.getDeclaredConstructor().newInstance();
+        assertThat(type.getMethod("nonCapturing").invoke(instance), notNullValue(Object.class));
+    }
+
     @Test(expected = IllegalStateException.class)
     public void testThrowExceptionIfNoMatch() throws Exception {
         new ByteBuddy()
