@@ -24,7 +24,10 @@ import java.util.Collection;
 import java.util.Enumeration;
 
 import static junit.framework.TestCase.assertEquals;
-import static org.hamcrest.CoreMatchers.*;
+import static org.hamcrest.CoreMatchers.is;
+import static org.hamcrest.CoreMatchers.not;
+import static org.hamcrest.CoreMatchers.notNullValue;
+import static org.hamcrest.CoreMatchers.nullValue;
 import static org.hamcrest.MatcherAssert.assertThat;
 import static org.mockito.ArgumentMatchers.anyString;
 import static org.mockito.ArgumentMatchers.eq;
@@ -33,9 +36,7 @@ import static org.mockito.Mockito.when;
 @RunWith(Parameterized.class)
 public class ByteArrayClassLoaderTest {
 
-    private static final ProtectionDomain DEFAULT_PROTECTION_DOMAIN = null;
-
-    private static final String FOO = "foo", BAR = "bar", QUX = "qux", CLASS_FILE = ".class";
+    private static final String FOO = "foo", BAR = "bar", QUX = "qux", CLASS_FILE = ClassFileLocator.CLASS_FILE_EXTENSION;
 
     private final ByteArrayClassLoader.PersistenceHandler persistenceHandler;
 
@@ -75,7 +76,7 @@ public class ByteArrayClassLoaderTest {
         classLoader = new ByteArrayClassLoader(ClassLoadingStrategy.BOOTSTRAP_LOADER,
                 false,
                 ClassFileLocator.ForClassLoader.readToNames(Foo.class),
-                DEFAULT_PROTECTION_DOMAIN,
+                ClassLoadingStrategy.NO_PROTECTION_DOMAIN,
                 persistenceHandler,
                 packageDefinitionStrategy,
                 classFilePostProcessor);
