@@ -16,7 +16,10 @@ import java.lang.reflect.Field;
 import java.lang.reflect.Method;
 
 import static net.bytebuddy.matcher.ElementMatchers.isDeclaredBy;
-import static org.hamcrest.CoreMatchers.*;
+import static org.hamcrest.CoreMatchers.instanceOf;
+import static org.hamcrest.CoreMatchers.is;
+import static org.hamcrest.CoreMatchers.not;
+import static org.hamcrest.CoreMatchers.sameInstance;
 import static org.hamcrest.MatcherAssert.assertThat;
 
 public class MethodDelegationOriginTest {
