@@ -4,14 +4,26 @@ import net.bytebuddy.ByteBuddy;
 import net.bytebuddy.description.type.TypeDescription;
 import net.bytebuddy.dynamic.loading.ClassLoadingStrategy;
 import net.bytebuddy.implementation.FixedValue;
+import net.bytebuddy.pool.TypePool;
 import net.bytebuddy.test.utility.JavaVersionRule;
 import org.hamcrest.CoreMatchers;
 import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.MethodRule;
+import org.objectweb.asm.ConstantDynamic;
+import org.objectweb.asm.Handle;
+import org.objectweb.asm.Opcodes;
+import org.objectweb.asm.Type;
+
+import java.util.Collections;
 
 import static net.bytebuddy.matcher.ElementMatchers.isDeclaredBy;
-import static org.hamcrest.CoreMatchers.*;
+import static org.hamcrest.CoreMatchers.equalTo;
+import static org.hamcrest.CoreMatchers.instanceOf;
+import static org.hamcrest.CoreMatchers.is;
+import static org.hamcrest.CoreMatchers.not;
+import static org.hamcrest.CoreMatchers.nullValue;
+import static org.hamcrest.CoreMatchers.sameInstance;
 import static org.hamcrest.MatcherAssert.assertThat;
 
 public class JavaConstantDynamicTest {
@@ -21,6 +33,27 @@ public class JavaConstantDynamicTest {
     @Rule
     public MethodRule javaVersionRule = new JavaVersionRule();
 
+    @Test
+    public void testAsmDynamicConstant() throws Exception {
+        assertThat(JavaConstant.Dynamic.ofAsm(TypePool.Default.ofSystemLoader(), new ConstantDynamic(
+                        FOO,
+                        Type.getDescriptor(Foo.class),
+                        new Handle(Opcodes.H_INVOKESTATIC,
+                                Type.getInternalName(SampleClass.class),
+                                FOO,
+                                Type.getMethodDescriptor(SampleClass.class.getMethod(FOO)),
+                                false), BAR)),
+                is(new JavaConstant.Dynamic(FOO,
+                        TypeDescription.ForLoadedType.of(Foo.class),
+                        new JavaConstant.MethodHandle(
+                                JavaConstant.MethodHandle.HandleType.INVOKE_STATIC,
+                                TypeDescription.ForLoadedType.of(SampleClass.class),
+                                FOO,
+                                TypeDescription.ForLoadedType.of(void.class),
+                                Collections.<TypeDescription>emptyList()),
+                        Collections.singletonList(JavaConstant.Simple.wrap(BAR)))));
+    }
+
     @Test
     @JavaVersionRule.Enforce(11)
     public void testDynamicConstantFactoryLookupOnly() throws Exception {
